package app.mar.activities;

import android.app.Activity;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.util.Timer;
import java.util.TimerTask;

import app.mar.game.Area;
import app.mar.game.Game;
import app.mar.game.Resource;
import app.mar.game.Stage;
import app.mar.game.challenges.Challenge;
import app.mar.game.challenges.Indice;
import app.mar.utils.AndroidResources;
import app.mar.utils.L;


public class HostActivity extends Activity implements ChallengeDriver{
    protected Game mGame;
    protected TextView mUITimeLeft;
    protected TextView mUICode;
    protected Button   mUIButtonVideos;
    protected Button   mUIButtonIndice;
    protected Button   mUIButtonValid;
    private Timer mTimer;

    protected QCMFragment mQCMFragment;
    protected QRCodeFragment mQRCodeFragment;
    protected QuestionFragment mQuestionFragment;
    protected ChallengeFragment mCurrentFragment;
    protected boolean isFirstRessrouceViewed=true;

    private static int sViewerDepth=0;

    public static void decViewerDepth() {  sViewerDepth--;   }
    public static void incViewerDepth() {  sViewerDepth++;   }
    private static final int ACTIVITY_RESULT_RESOURCE_VIEWER = 1;

    private Runnable mTimerTick = new Runnable() {
        public void run() {
            if(!mGame.getBomb().hasExplosed())
            {
                mUITimeLeft.setText(mGame.getBomb().getTimeBeforeBoom());
            }else
            {
                mUITimeLeft.setText("BOOM !");
                onExplose();
                mTimer.cancel();
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_host);

        mQCMFragment = QCMFragment.newInstance();
        mQuestionFragment = QuestionFragment.newInstance();
        mQRCodeFragment = QRCodeFragment.newInstance();
        mUIButtonVideos = (Button) findViewById(R.id.btn_videos);
        mUIButtonIndice = (Button) findViewById(R.id.btn_indice);
        mUIButtonValid = (Button) findViewById(R.id.btn_valid);

        mGame=Game.game();


        mUITimeLeft = findViewById(R.id.tv_car);
        mUICode = findViewById(R.id.tv_code);

        mTimer = new Timer();
        mTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                timerCallback();
            }

        }, 0, 1000);

        updateCode();

        if(mGame.isFirstChallenge()) {
            mGame.getCurrentStage().execStartEvent(this, mGame);
        }
        startChallenge();

    }

    public   void setFragment(Class type)
    {
        ChallengeFragment f = null;

        if(type.isInstance(mQCMFragment))
            f= mQCMFragment;
        else if(type.isInstance(mQRCodeFragment))
            f= mQRCodeFragment;
        else if(type.isInstance(mQuestionFragment))
            f= mQuestionFragment;
        else {
            throw new RuntimeException("Erreur: La classe de fragment est null");
        }

        if(f!=mCurrentFragment)
        {
            mCurrentFragment=f;
            FragmentManager fm = getFragmentManager();
            FragmentTransaction ft = fm.beginTransaction();
            ft.replace(R.id.container, mCurrentFragment);
            ft.commit();
        }
        else
        {
            onReady();
        }
    }

    @Override
    public void onReady() {
        Challenge chall = mGame.getCurrentChallenge(this);
        mCurrentFragment.initChallenge(chall);
    }



    @Override
    public double getDistanceWith(Area a) {
        return a.getDistance(mGame.getPlayer());
    }

    @Override
    public double getAccuracy() {
        return mGame.getPlayer().getSensorsManager().getPosition().getAccuracy();
    }


    @Override
    public void onFragmentInteraction(Uri uri) {

    }


    private void timerCallback()
    {
        this.runOnUiThread(mTimerTick);
    }

    private void updateCode()
    {
        Log.e("--------", "Bomb: "+mGame.getBomb().getPasswordKnown());
        mUICode.setText(mGame.getBomb().getPasswordKnown());
    }

    public void onClickValid(View v)
    {
        if(mCurrentFragment!=null)
        {
            if(mCurrentFragment.check())
            {
                challengeValidate(mCurrentFragment.getChallenge(),  mGame, this);
            }
        }
    }

    public void onClickVideos(View v)
    {
        Intent intent = new Intent(this, ResourceListActivity.class);
        intent.putExtra("inventaire", mGame.getInventaire());
        startActivity(intent);
    }

    public void onClickDisarm(View v)
    {

    }

    public void onClickIndice(View v)
    {
        if(mCurrentFragment!=null)
        {
            Indice ind = mCurrentFragment.showIndice();
            if(ind!=null && ind.getPenality()>0)
                mGame.getBomb().addTime(-ind.getPenality());

            if(!mCurrentFragment.getChallenge().hasNextIndice())
                mUIButtonIndice.setVisibility(View.INVISIBLE);
        }
    }

    public void onExplose()
    {
        setFailed();
    }

    public void setVisibilityView(int res, int state)
    {
        View v = findViewById(res);
        if(v!=null) v.setVisibility(state);
    }

    public void setVisibilityView(int[] res, int state)
    {
        for(int i=0; i<res.length; i++)
            setVisibilityView(res[i], state);
    }

    public void nextChallenge() {
        Stage s = mGame.getCurrentStage();

        //on lance levenemnt de fin du challenge precedent
        if(mGame.getCurrentChallenge(this)!=null && !mGame.isFirstChallenge())
            mGame.getCurrentChallenge(this).execEndEvent(this, mGame);

        Challenge c = s.nextChallenge(this, mGame);
        if(c!=null)
        {
            startChallenge();
            Log.i("----", "Continue challenge in stage");
        }else //new stage
        {
            Log.i("----", "New Stage !");
            mGame.getBomb().addKnownCharacter(s.getCharIndex());
            updateCode();
            if(mGame.nextStage(this)!=null) { // il reste au moins une étape
                startChallenge();
            }
            else //c'est fini
            {
                startResourceFound();
                //setFinished();
            }

        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == ACTIVITY_RESULT_RESOURCE_VIEWER) {
            startResourceFound();
        }
    }

    private boolean startResourceFound()
    {
        if(isFirstRessrouceViewed && !mGame.isFirstChallenge())
        {
            isFirstRessrouceViewed=false;
            if( mGame.getCurrentChallenge(this)!=null)
                mGame.getCurrentChallenge(this).execStartEvent(this, mGame);
        }
        Resource r = mGame.popLastResource();

        if(r!=null )
        {
            Intent intent = AndroidResources.getViewerIntent(this, r);
            incViewerDepth();
            startActivityForResult(intent, ACTIVITY_RESULT_RESOURCE_VIEWER);
            return true;
        }else //plus de ressources a montrer
        {
            Challenge c = mGame.getCurrentChallenge(this);
            if(c!=null) c.execStartEvent(this, mGame);
            isFirstRessrouceViewed=true;
            return false;
        }


    }

    @Override
    public void challengeValidate(Challenge c, Game g, ChallengeDriver dri) {
        c.setDone(dri, g);
        mGame.getBomb().addKnownCharacter(c.getCharIndex());
        updateCode();
        nextChallenge();
    }

    public Challenge currentChallenge() {
        return mGame.getCurrentChallenge(this);
    }

    public void setFinished()
    {
        Intent intent = new Intent(this, CreditsActivity.class);
        startActivity(intent);
        finish();
    }

    public void setFailed()
    {
        Intent intent = new Intent(this, CreditsActivity.class);
        startActivity(intent);
        finish();
    }

    private boolean startChallenge()
    {
        boolean b=false;
        int res[] = {R.id.btn_indice, R.id.btn_valid, R.id.btn_videos};
        for(int i=0; i<res.length; i++)
        {
            View v = findViewById(res[i]);
            v.setVisibility(View.VISIBLE);
        }

        startResourceFound();

        Challenge c = mGame.getCurrentChallenge(this);

        if(c==null) //fin du stage
        {
            mGame.nextStage(this);
        }

        if(c!=null) {
            c.exec(mGame,this);
            b=true;
        }

        nextChallenge();
        return b;
    }

    @Override
    public void startLocalisation() {
        mGame.newSensorManager(this);
    }

    @Override
    public void resumeLocalisation() {
        mGame.onResume();
    }

    @Override
    public void pauseLocalisation() {
        mGame.onPause(this);
    }

    @Override
    public void onBackPressed()
    {
        if(mCurrentFragment==null || mCurrentFragment.onBackPressed())
            super.onBackPressed();
    }
}